/***************************************************
 * pliboo-asservissement V-00-002 Rev-001
 * Février 2017
 * Fait pour le projet PLIBOO 
 * Prototype de petite éolienne
 * PLIBOO est développé sous licence CERN OHL
 *
 * DESCRIPTION / OBJET
 * Ce programme arduino, a pour fonction, 
 * de piloter avec une arduino uno la tension qui 
 * vient d'une mini éolienne pouvant produire des 
 * tensions variables de 0 à 240 volts, ou au delà.
 *
 * Nota:
 * Après le code ci-dessous, des explications sont
 * données par un système de renvoit de notes
 ***************************************************
 */

/* ######################################################################
 * PARTIE A. LE CODE 
 * ######################################################################
 */
	// Note 000
	const int EntreeCommandeTension = A0; // Note 001
	const int SortieCommandeTransistor01 = 2; // Note 002
	const int SortieCommandeTransistor02 = 3; // Note 002

	const float diviseur = 74.93; // Note 023
	const float CoefAnalogique = (5.0 / 1023.0); // Note 024
	const float EquivEol = CoefAnalogique * diviseur; // Note 025
	const float PalierVoltage = 24; // Note 026
	const int intervalleDeLecture = 100; // Note 027
	
	int ValeurAnalog = 0; // Note 041
	float Ueol = 0; // Note 042
	int TransistorNum01Etat = LOW; // Note 043
	int TransistorNum02Etat = LOW; // Note 044

	// Note 100
	void setup() { 
 		Serial.begin(9600); // Note 101
		pinMode(EntreeCommandeTension, INPUT); // Note 102
		pinMode(SortieCommandeTransistor01, OUTPUT); // Note 103
		pinMode(SortieCommandeTransistor02, OUTPUT); // Note 103

		}
	
	// Note 200
	void loop() { 
	       	ValeurAnalog = analogRead(A0); // Note 201  EntreeCommandeTension
 		Ueol = ValeurAnalog * EquivEol; // Note 202
		// Note 250		
		if 	(Ueol < 1) 
			{
			TransistorNum01Etat = LOW;
			TransistorNum02Etat = LOW;
			} 

		else if	( (Ueol >= 1) && (Ueol <= PalierVoltage) ) 
			{
			TransistorNum01Etat = HIGH;
			TransistorNum02Etat = LOW;
			} 

		else if ( (Ueol >= PalierVoltage) && (Ueol <= (2 * PalierVoltage) )  )
			{
			TransistorNum01Etat = LOW;
			TransistorNum02Etat = HIGH;
 			}	
		else 	{
			TransistorNum01Etat = LOW;
			TransistorNum02Etat = LOW;
 			}
		digitalWrite(SortieCommandeTransistor01, TransistorNum01Etat);
		digitalWrite(SortieCommandeTransistor02, TransistorNum02Etat);
		delay(intervalleDeLecture); 
		Serial.print("Ueol: "); // Note 203
		Serial.println(Ueol); // Note 203
		}



/* ######################################################################
 * PARTIE B. LES EXPLICATIONS 
 * ######################################################################
 */

/* NOTE 001: DÉFINITIONS */

/* NOTE 001: SÉLECTION DE LA FICHE ANALOGIQUE DE COMMANDE
 * Sélection de la fiche d'entrée analogique dans l'arduino 
 * du fil provenant du diviseur de tension
 * et qui va être l'indicateur de tension
 * L'une des fiches A0 à A5
 * const int EntreeCommandeTension = A0;
 */

/* NOTE 002 : SÉLECTION DE LA FICHE DE SORTIE POUR COMMANDER LE TRANSISTOR 1
 * Sélection de la fiche qui connecte l'arduino aux
 * transistors PN2222, commandés par l'arduino
 * L'une des fiches 0 à 13 convient.
 * const int SortieCommandeTransistor01 = 1;
 * const int SortieCommandeTransistor02 = 2;
 */

/* NOTE 023 : DIVISION DE TENSION EN ENTRÉE
 * On place un pont diviseur fait de deux résistance R1 et R2
 * Après mesure, on détecte que ce pont diviseur divise par 
 * 74,93
 * On appelle ce nombre diviseur: diviseur
 * Cela signifie que pour 1 volt sortant du pont diviseur
 * on a 74,93 volts livrés par l'éolienne
 * const float diviseur = 74.93;
 */      

/* NOTE 024 : ÉQUIVALENCE VOLTS ET UNITÉS ANALOGIQUES DANS LA FICHE ANALOGIQUE
 * La fiche analogique de l'Arduino, lit des unités comprises entre 0 et 1023
 * Lorsque qu'il y a 5 volts qui entrent dans la fiche, l'arduino affecte
 * 1023 unités analogiques: 1023 unités -> 5 volts
 * Et donc, par règle de trois 1 unité analogique correspond à (5/2013) volt, 
 * 1 unité - > (5/2013)=0,00488 volt
 * On donne un nom à ce coefficient: CoefAnalogique
 * Attention !: il faut un espace entre le symbole division et les chiffres !!!
 * const float CoefAnalogique = (5.0 / 1023.0);
 */

/* NOTE 025: ÉQUIVALENCE ENTRE FICHE ANALOGIQUE ET LE VOLTAGE DE L'ÉOLIENNE
 * 1 unité analogique correspond à (5/2013) volt dans la fiche analogique
 * et 1 volt analogique correspond à 74.93 volts de l'éolienne
 * Donc:
 * 1 unit analog -> (5/1023) volt analog -> (5/1023)x74.93 volts éolienne
 * Donc 1 unit analog -> (CoefAnalogique) * (diviseur) volts éolienne
 * On nomme ce rapport EquivEol
 * float EquivEol = (CoefAnalogique) * (diviseur);
 */

/* NOTE 026: DÉFINITION DES TRANCHES DE PALIERS DE VOLTAGE
 * On défini les tranches de paliers de voltage déclencheurs
 * des transistors
 * const int PalierVoltage = 24;
*/

/* NOTE 27: DÉFINITION DES INTERVALLES DE LECTURE
 * On lit les valeurs avec un délai plus ou moins grand
 * 1000 signifie 1000 millisecondes, soit 1 seconde
 * 100 signifie 1 dizième de seconde (O,1 seconde)
 * const int intervalleDeLecture = 100; 
 */

/* NOTE 041: MISE À ZÉRO DE LA VALEUR ANALOGIQUE
 * On nomme la valeur analogique qui sera lue ValeurAnalog
 * Cette valeur est mise à zéro en début de programme
 * int ValeurAnalog = 0;
 */

/* NOTE 042: MISE À ZÉRO DE LA VALEUR DE VOLTAGE ÉOLIENNE
 * On nomme le voltage fourni par l'éolienne Ueol.
 * Cette valeur est mise à zéro en début de programme
 * float Ueol = 0;
 */

/* NOTE 043: ETAT DE DÉPART DES TRANSISTORS COMMANDÉS 
 * Les transistors ne doivent pas faire passer le courant (Etat = LOW)
 * tant qu'on ne leur dit pas le contraire
 * int TransistorNum01Etat = LOW; 
 * int TransistorNum02Etat = LOW; 
*/

/* NOTE 100: INITIALISATION 
 * initialise les paramètres
 * void setup() {}
 */

/* NOTE 101: 
 * On initialise la routine sur le port série à 9600 baux par seconde
 * Serial.begin(9600);
*/

/* NOTE 102: 
 * On dit à l'Arduino que le fiche ANALOGIQUE est une fiche d'entrée
 * de signal basé sur EntreeCommandeTension (voir note 001)
 * pinMode(EntreeCommandeTension, INPUT);
 */

/* NOTE 103: 
 * On dit à l'Arduino que le fiche SortieCommandeTransistor01 
 * est une fiche de sortie de signal (voir note 002)
 * pinMode(SortieCommandeTransistor01, OUTPUT);
 */

/* NOTE 200: BOUCLE
 * On lance le programme sous forme de boucle
 * On fait une boucle qui ne s'arrête jamais
 * void loop() {}
 */

/* NOTE 201: LECTURE DE LA VALEUR ANALOGIQUE
 * L'arduino lit la valeur analogique qu'il y a dans la fiche analogique
 * ValeurAnalog = analogRead(EntreeCommandeTension); 
 */ 

/* NOTE 202: TRANSFORMATION DE LA VALEUR ANALOGIQUE EN VOLTAGE ÉOLIENNE
 * On transforme la valeur qui est dans la fiche analogique
 * en voltage fourni par l'éolienne, en multipliant la valeur analogique
 * par le rapport EquivEol (voir Note 05)
 * float Ueol = ValeurAnalog * EquivEol;
 */

/* NOTE 203: AFFICHAGE DANS LE MONITEUR SÉRIE 
 * L'arduino affiche la valeur du voltage de l'éolienne dans le 
 * moniteur série (menu -> outils -> moniteur série)
 * Serial.println(Ueol);
 */

